/*
===========================================================
X. Bouillard   2014/08/01
Commande de pont tournant avec acceleration et decelleration 

Moteur Unipolaire 5V 400 pas (0.9°) RH7-1336
ou     Bipolaire  5V                D28BYJ-48
ou     Bipolaire 12V                42BYGH44958A         (Defaut)
EasyDriver V44 double le nombre de pas
DRV V44 multiplie le nombre de pas par 32                (Defaut)
Clignottement led
Clavier matriciel
RC5
Decodeur DCC (developpement en cours)
===========================================================
? : Affichage du menu
E : passage en mode Exploitation
S : passage en mode Reglage
  Permet d'ajuster le positionnement du pont face a une voie 
  (indiquer le pas apres le S)
> : Next
  Passe a la voie suivante (Exploitation)
  Avance d'un micro pas (Reglage)
< : Prev
  Passe a la voie precedente (Exploitation)
  Recule d'un micro pas (Reglage)
R : Retour = demi tour
Z : Zero, La voie actuelle devient la voie Zero 
===========================================================
Recepteur RC5
 ------
|      |----- RC
| TK69 |----- +5v
|      |----- Gnd
 ------
===========================================================
Compatibilite DCC Fleischman:
Chaque voie correspond a un bit qui monte pendant 1/2 seconde
Pour 40 voies il faut donc 5*8 bits soit 5 adresses successives

Compatibilité Marklin:
Chaque voie correspond a deux bits qui monte pendant 1/2 seconde comme pour un aiguillage
*/

// ========================================================
// Gestion de la communication PC via USB
String inputString = "";        // Chaine permettant de recevoir les informations du PC via USB
boolean stringComplete = false; // Drapeau indiquant que la chaine recue est complete (finit par CR)

#include <AccelStepper.h>
#include <Keypad.h>
#include <IRremote.h>
#include <DCC_Decoder.h>
#include <EEPROM.h>

#define  Feu  11              // Led feu pour test
// ========================================================
// Definition des messages
#define  Msg_Patience          "Patientez S.V.P."
#define  Msg_Done              "En "

#define  Msg_Next              "Next"
#define  Msg_Prev              "Prev"
#define  Msg_Half              "Retour"
#define  Msg_Zero              "Zero"
#define  Msg_Goto              " Goto  :"
#define  Msg_Adr               " Adresse "
#define  Msg_Vitesse           " Vitesse "
#define  Msg_Track             " 1/2 Voie "
#define  Msg_Pas               " Pas "
#define  Msg_Kind              " Type de pont "
#define  Msg_List              " Parametres"

#define  Msg_Exploit           "Exploitation"
#define  Msg_Setting           "Reglage"

#define  Msg_Move              "Move "
#define  Msg_Motor             " Cmd moteur "
#define  Msg_Fade              "Fade "

#define  Cmd_Help              "?" // pour Aide
#define  Msg_Help              "Aide" // pour Aide

#define  Msg_Id                "Who's Id "

#define  Err_Low               "Err <0"
#define  Err_High              "Err >"
#define  Err_Msg               "???"

#define  Msg_Recu_USB           "------ Recu :"
#define  Msg_Recu_Kbd           "------ Lu   :"
#define  Msg_Recu_RC            "------ RC   :"

#define  Cmd_Exploit           "E" // pour Exploitation
#define  Cmd_Setting           "S" // pour Setting
#define  Cmd_Next              ">" 
#define  Cmd_Prev              "<"
#define  Cmd_Half              "R" // pour Retournement
#define  Cmd_Zero              "Z" // pour Zero
#define  Cmd_Fade              "F" // pour Fade
#define  Cmd_Adr               "A" // pour Adresse
#define  Cmd_Vitesse           "M" // pour Maximum
#define  Cmd_Track             "T" // pour Track
#define  Cmd_Pas               "P" // pour Pas
#define  Cmd_Kind              "K" // pour Kind
#define  Cmd_Help              "?" // pour Aide
#define  Cmd_List              "L" // pour Liste
#define  Cmd_Id                "W" // pour Aide
#define  Msg_TT                "TT Dcc Ard"

// Adaptation du materiel
int HalfTrack=20;              // Nombre de voies /2
int Kind=0;                    // Type de pont 0=Tournant 1=Transbordeur
#define  DIR_PIN     8         // Commande du sens
#define  STEP_PIN    9         // Commande de marche/arret 
#define  Reset_PIN  10         // Commande de marche/arret 
#define  interval  500         // Timeout detection moteur
                               // A regler sur 110% du demi tour
// Specific RH7-1336
//#define r9         40          // Nombre de pas entre voies
//#define MaxSpeed   30          // Vitesse maxi
//#define Accel      50          // Acceleration

// Specific 28BYJ-48
//#define r9         120         // Nombre de pas entre voies
//#define MaxSpeed   500         // Vitesse maxi
//#define Accel      1000        // Acceleration

// Specific SY28STH45-0674A
//#define r9         40          // Nombre de pas entre voies
//#define MaxSpeed   25          // Vitesse maxi
//#define Accel      50          // Acceleration

// Specific 42BYGH44958A
int settingStep=1;               // Nombre de pas au reglage 
int runingStep=320;              // Nombre de pas entre voies
int MaxSpeed=100;                // Vitesse maxi
int DccBaseAdress=10;            // Adresse Dcc
#define Accel        50          // Acceleration

#define NoMvt        -29999      // Empeche le Mvt
#define led1         12
#define led2         13
#define BlkInterval  1000        // Frequence de clignottement
#define Alt          true        // Alterne le clignottement

// Keyboard
#define Kbd_Timeout  3000        // Timeout Clavier
const byte ROWS = 4;             //four rows
const byte COLS = 3;             //three columns
//char hexaKeys[ROWS][COLS] = {
//  {'0','1','2','3'},
//  {'4','5','6','7'},
//  {'8','9','A','B'},
//  {'C','D','E','F'}
//};
char keys[ROWS][COLS] = {
    {'1','2','3'},
    {'4','5','6'},
    {'7','8','9'},
    {'*','0','#'}
};

byte rowPins[ROWS] = {4, 5, 6, 7}; //connect to the row pinouts of the keypad
byte colPins[COLS] = {A3, A2, A1}; //connect to the column pinouts of the keypad

Keypad keypad = Keypad( makeKeymap(keys), rowPins, colPins, ROWS, COLS );
byte ledPin = 13; 

boolean blink = false;
boolean ledPin_state;

#define FadeInterval 5           // Frequence fade
#define FadeStep 5               // Step fade
long LastFade =0;
boolean Feu_state;
int Feu_Value;

// RC5 receiver
#define  Rc_Pin                3 // remplacement de 11  // Code  RC
IRrecv reception_ir(Rc_Pin);
decode_results decode_ir;
int RC=-1;
int cpt;

// Define a stepper and the pins it will use
AccelStepper stepper(1, STEP_PIN, DIR_PIN);

int pos = 0;            // Pas du moteur 0 à 12800;
boolean Connected=true;
boolean Done;
int rot = 1;            // sens de rotation
int voie=0;             // Numero de la voie principale
int mode=0;             // Exploitation=0, Reglage =1
long previousMillis = 0;
unsigned long currentMillis = millis();
int tmp=NoMvt;

int ledState = LOW;     // ledState used to set the LEDs
long LastBlk = 0;
String KbdStr; 
unsigned long startTime;

String RcStr;
String UsbStr;
int LastMsg =0;
int LastCmd =0; 

int Eeprom = 0;    // 0 à 512 disponible en EEPROM

#define kDCC_INTERRUPT            0

typedef struct
{
    int               address;                // Address to respond to
//    byte              output;                 // State of output 1=on, 0=off
//    int               outputPin;              // Arduino output pin to drive
//    boolean           isDigital;              // true=digital, false=analog. If analog must also set analogValue field
//    boolean           isFlasher;              // true=flash output, false=no time, no flash.
//    byte              analogValue;            // Value to use with analog type.
    int               durationMilli;          // Milliseconds to leave output on for.  0 means don't auto off
    byte              DataValue;       // Value sent.
    
    unsigned long     onMilli;                // Used internally for timing
    unsigned long     offMilli;               // 
} DCCAccessoryAddress;

DCCAccessoryAddress gAddresses[8];

// ========================================================
// DCC Init
// ========================================================
void ConfigureDecoder()
{
// Compatibilite Fleischmann
// Chaque voie correspond a un bit
    int i=0;
    for(int i=0; i< HalfTrack*2; i++) {
/*
        Serial.print(i);
        Serial.print(":");
        Serial.print(AdresseDcc+i/8);
        Serial.print("-");
        Serial.println(i%8);
*/
        gAddresses[i].address = DccBaseAdress+i-4;
        gAddresses[i].durationMilli = 0; //if a servo is coupled set this at 0! 
        gAddresses[i].DataValue=0;
    }
}

// ========================================================
// Basic accessory packet handler 
// ========================================================
void BasicAccDecoderPacket_Handler(int address, boolean activate, byte data) {
    // Convert NMRA packet address format to human address
    address -= 1;
    address *= 4;
    address += 1;
    address += (data & 0x06) >> 1;
    
    boolean enable = (data & 0x01) ? 1 : 0;

    int msg=data;
    if (LastMsg != msg) { 
 /*
      Serial.print("S ");
      Serial.print(address-DccBaseAdress);
      Serial.print(" (");
      Serial.print(address);
      Serial.print(") ");
      Serial.println(enable,DEC);        
      Serial.print(" data: ");
      Serial.println(data,BIN);    
 */    
      LastMsg = msg;
    }
    for(int i=0; i< HalfTrack*2; i++) {
      if( address == gAddresses[i].address ) {
        if( enable ) {
          //gAddresses[i].output = 1;
          //gAddresses[i].onMilli = millis();
          //gAddresses[i].offMilli = 0;
          tmp=Goto(address-DccBaseAdress+4);
          Serial.print(" Dcc Goto: ");
          Serial.println(tmp); 
        } else {
          //gAddresses[i].output = 0;
          //gAddresses[i].onMilli = 0;
          //gAddresses[i].offMilli = millis();
        }
        gAddresses[i].DataValue=data;
      }
   }
    
//    for(int i=0; i<(int)(sizeof(gAddresses)/sizeof(gAddresses[0])); i++) {
//        if( address == gAddresses[i].address ) {
//            Serial.print("Basic addr: ");
//            Serial.print(address,DEC);
//            Serial.print("   activate: ");
//            Serial.println(enable,DEC);
//            Serial.print(" => voie ");
//            Serial.println((address-AdresseDcc)*8+enable,DEC);        
//        }
//    }   
}

// ========================================================
// Lecture USB Serial event reader
// ========================================================
void serialEvent() {
  while (Serial.available()) {
    // acquisition d'un nouveau caratere:
    char inChar = (char)Serial.read(); 
    // si c'est LF, positionne le drapeau pour que la boucle principale prenne en compte le message:
    if (inChar == '\n') {
      stringComplete = true;
    } else {
    // sinon l'ajouter au message en cours de construction:
      inputString += inChar;
    }
  }
}

// ========================================================
void Help() {
  Serial.print(Cmd_Exploit);Serial.print(" : "); Serial.println(Msg_Exploit);
  Serial.print(Cmd_Setting);Serial.print(" : "); Serial.println(Msg_Setting);
  Serial.print(Cmd_Next);Serial.print(" : "); Serial.println(Msg_Next);
  Serial.print(Cmd_Prev);Serial.print(" : "); Serial.println(Msg_Prev);
  Serial.print(Cmd_Half);Serial.print(" : "); Serial.println(Msg_Half);
  Serial.print(Cmd_Zero);Serial.print(" : "); Serial.println(Msg_Zero);
  Serial.print(Cmd_Fade);Serial.print(" : "); Serial.println(Msg_Fade);
  Serial.print(Cmd_List);Serial.print(" : "); Serial.println(Msg_List);
  Serial.print(Cmd_Adr);Serial.print(" : "); Serial.println(Msg_Adr);
  Serial.print(Cmd_Vitesse);Serial.print(" : "); Serial.println(Msg_Vitesse);
  Serial.print(Cmd_Track);Serial.print(" : "); Serial.println(Msg_Track);
  Serial.print(Cmd_Pas);Serial.print(" : "); Serial.println(Msg_Pas);
  Serial.print(Cmd_Kind);Serial.print(" : "); Serial.println(Msg_Kind);

  Serial.print(Cmd_Help);Serial.print(" : "); Serial.println(Msg_Help);
  Serial.print(Cmd_Id);Serial.print(" : "); Serial.println(Msg_Id);
  Serial.println();
}

// =====================================================
// Lecture CV
// =====================================================
long GetCV(int Ad) {
  long v;
  v=EEPROM.read(Ad);
  v=100*v+EEPROM.read(Ad+1);
  return v;
}

// =====================================================
// Ecriture CV
// =====================================================
void SetCV(String s, int Ad) {
  Serial.println(s);
  s="0000"+s;
  s=s.substring(s.length()-5);
  EEPROM.write(Ad, s.substring(0,2).toInt());
  EEPROM.write(Ad+1, s.substring(2).toInt());
}

// ========================================================
// Avance du moteur de d pas
// ========================================================
void MoveMotor(int d) {
  if (Connected) {
    stepper.move(d);
    previousMillis = currentMillis;
  }
}

// ========================================================
// Verifie que la voie demandée est valide, retourne pos=nb de pas
// ========================================================
int Goto(int t) {
  if (t<0) {                  // Voie non disponible <0
    Serial.print(Err_Low);
  t=NoMvt;
  } else if (t<=2*HalfTrack) {
    int t1;
    if (abs(t-voie)<=HalfTrack) {
      t1=t-voie;
    } else {
      t1=abs(voie-t)-2*HalfTrack;
      if (voie>t) {
        t1=-t1;
      }
    }
    pos=t1;
    Serial.print(Msg_Move);
    Serial.print(t1);
    Serial.println(Msg_Goto);
  } else if (t>2*HalfTrack){ // Voie non disponible > max
    Serial.print(Err_High);
    Serial.print(2*HalfTrack);
    t=NoMvt;
  } 
  return t;
}

// ========================================================
void setup() {
  Serial.begin(9600);              // initialise la communication USB:
  Serial.println(Msg_Patience);
  Init();
  List();
}

// ========================================================
void Init() {
  // Lecture de parametres enregistrés en EEPROM
  DccBaseAdress = GetCV(Eeprom);      // Initialise l'adresse DCC
  HalfTrack = GetCV(Eeprom+2);     // Initialise le nombre de voies
  MaxSpeed = GetCV(Eeprom+4);      // Initialise la vitesse maximum
  runingStep = GetCV(Eeprom+6);    // Initialise le nombre de pas entre voies
  Kind = GetCV(Eeprom+8);          // Initialise le type de pont
  Serial.print(Msg_Done);
  Serial.println(voie);
        
  inputString.reserve(100);        // reserve 100 bytes pour la chaine en reception USB:
  pinMode(Feu, OUTPUT);            // Led Feu pour test
  
  pinMode(led1, OUTPUT);
  pinMode(led2, OUTPUT);
  pinMode(Reset_PIN, OUTPUT);
  digitalWrite(Reset_PIN,LOW);
  delay(1);
  digitalWrite(Reset_PIN,HIGH);
  
//  keypad.addEventListener(keypadEvent); // Add an event listener for this keypad
  Keypad keypad = Keypad( makeKeymap(keys), rowPins, colPins, ROWS, COLS); 

// Stepper Setup
  stepper.setMaxSpeed(MaxSpeed);
  stepper.setAcceleration(Accel);
  Help();
  Serial.println(Msg_Exploit);

// RC Setup
  reception_ir.enableIRIn(); // Demarre la reception RC
  
// Dcc Setup
   DCC.SetBasicAccessoryDecoderPacketHandler(BasicAccDecoderPacket_Handler, true);
   ConfigureDecoder();
   DCC.SetupDecoder( 0x00, 0x00, kDCC_INTERRUPT );
}

// ========================================================
// Liste les parametres
// ========================================================
void List() {
   Serial.print(Msg_Adr);
   Serial.println(DccBaseAdress);
   Serial.print(Msg_Vitesse);
   Serial.println(MaxSpeed);
   Serial.print(Msg_Track);
   Serial.println(HalfTrack);
   Serial.print(Msg_Pas);
   Serial.println(runingStep);
   Serial.print(Msg_Kind);
   Serial.println(Kind);
}

// ========================================================
// Boucle principale
// ========================================================
void loop() {
//  int tmp=NoMvt;
// ......................
// Gestion des info DCC
  static int addr = 0;
  static long S;
  DCC.loop(); 
/*  
  // Saute a l'adresse suivante
  if( ++addr >= (int)(sizeof(gAddresses)/sizeof(gAddresses[0])) )  {
    addr = 0;
  }
  if( gAddresses[addr].offMilli && gAddresses[addr].offMilli<millis() ) {
    // Clear off time
    gAddresses[addr].offMilli = 0;
    if (LastCmd!=gAddresses[addr].address) {
      Serial.print("Go to : ");
      Serial.println(gAddresses[addr].address-AdresseDcc+4);
      //Serial.print(" Data ");
      //Serial.println(gAddresses[addr].DataValue);
      LastCmd=gAddresses[addr].address;
      tmp=Goto(gAddresses[addr].address-AdresseDcc+4);
      Done=true;
    }  
    return;
  }

  //tmp=Goto(DccStr.toInt());
*/

// ......................
// Gestion des info kbd
  char key = keypad.getKey();
  if (key){
    switch (key) {
      case '#':                      //  ...................... ???
        Serial.print(Msg_Recu_Kbd);
        // Led Off
        Serial.print(Msg_Fade);
        Feu_state=!Feu_state;
        Serial.println(Feu_state);
        break;
      case '*':                      //  ...................... 1/2 tour
        Serial.print(Msg_Recu_Kbd);
        Serial.println(Msg_Half);
        pos=HalfTrack;
        tmp=(voie+HalfTrack)%(2*HalfTrack);
        Done=true;
        KbdStr="";
       break;
      default:                       //  ...................... Goto track
        if (KbdStr.length()==0) {
          Serial.print(Msg_Recu_Kbd);
          startTime = millis();
        } 
        Serial.print(key);
        KbdStr+=key;
    }
  }
// ......................
// Detection fin de kbd sur timeout ou 2 touches
  if ((millis()-startTime>Kbd_Timeout) || (KbdStr.length()==2)) {
    if (KbdStr!="") {
      Serial.println("");
      Serial.print(">>>");
      Serial.println(KbdStr);
      tmp=Goto(KbdStr.toInt());
      Done=true;
      //tmp=Goto(kbdStr.toInt());
      KbdStr="";
    }
  }   
// ......................
// Gestion des RC : 
  if (reception_ir.decode(&decode_ir)) {
    int t=millis()-startTime;
    if (t>500) {
      if (decode_ir.value>65535) {
        RC=decode_ir.value-65536; 
      } else {     
        RC=decode_ir.value;
      } 
      startTime = millis();
      switch (RC) {
//        case -1:                      //  ...................... ???
//          Serial.println("RAF");
//          break;
        case 90:                      //  ...................... ???
          Serial.print(Msg_Prev);
          pos=-1;
          tmp=(voie-1+2*HalfTrack)%(2*HalfTrack);
          Done=true;
          RcStr="";
          break;
        case 91:                      //  ...................... ???
          Serial.print(Msg_Next);
          pos=1;
          tmp=(voie+1)%(2*HalfTrack);
          Done=true;
          RcStr="";
          break;
        case 92:                      //  ...................... ???
          Serial.print(Msg_Half);
          pos=HalfTrack;
          tmp=(voie+HalfTrack)%(2*HalfTrack);
          Done=true;
          RcStr="";
          break;
        default:
          if (Done==false) {
            Serial.print(RC,HEX);
          }
          RcStr+=RC;
      }
    }
    reception_ir.resume();
  }
// ......................
// Detection fin de rc sur timeout ou 2 touches    
  if ((millis()-startTime>Kbd_Timeout) || (RcStr.length()==2)) {
    if (RcStr!="") {
      Done=true;
//      Serial.println();
//      Serial.print("B>>>");
//      Serial.println(RcStr);
      tmp=Goto(RcStr.toInt());
      RcStr="";
    }
  }      

// ......................
// Gestion des info USB : 
  if (stringComplete) {
    UsbStr="";
    Done=true;
    inputString.toUpperCase();
    Serial.print(Msg_Recu_USB);
    Serial.println(inputString);
    String c = inputString.substring(0,1);
       
    switch (mode) {
      case 0: // ...................... mode Exploitation
        if (c==Cmd_Help) {           // ...................... Aide
          Help();
        } else if (c==Cmd_Setting) { // ...................... Passe en Mode Settings
          Serial.println(Msg_Setting);
          settingStep=(inputString.substring(1).toInt());
          mode = 1;
          Serial.println(settingStep);
        } else if (c==Cmd_Next) {    // ...................... Next Step
          Serial.print(Msg_Next);
          //UsbStr=(voie+1)%(2*HalfTrack);
          pos=1;
          tmp=(voie+1)%(2*HalfTrack);
        } else if (c==Cmd_Prev) {    // ...................... Previous Step
          Serial.print(Msg_Prev);
          //UsbStr=(voie-1+2*HalfTrack)%(2*HalfTrack);
          pos=-1;
          tmp=(voie-1+2*HalfTrack)%(2*HalfTrack);
        } else if (c==Cmd_Half) {    // ...................... 1/2 tour
          Serial.print(Msg_Half);
          //UsbStr=(voie+HalfTrack)%(2*HalfTrack);
          pos=HalfTrack;
          tmp=(voie+HalfTrack)%(2*HalfTrack);
        } else if (c==Cmd_Fade){     // Led Off
          Serial.print(Msg_Fade);
          Feu_state=!Feu_state;
          Serial.println(Feu_state);
          Done=false;
        } else if (c==Cmd_Id) {         // ...................... On
          Serial.println(Msg_TT);
        } else if (c!=""){           // ...................... Goto track
//          tmp=(inputString.toInt()+2*HalfTrack)%(2*HalfTrack);
          tmp=Goto(inputString.toInt());
          Serial.print(" USB Goto: ");
          Serial.println(tmp); 
        }
        break;
      case 1:  // ...................... Mode reglage
        if (c==Cmd_Help) {           // ...................... Aide
          Help();
        } else if (c==Cmd_Exploit) { // ...................... Passe en Mode Exploitation
          mode = 0;
          Serial.println(Msg_Exploit);
        } else if (c==Cmd_Setting) { // ...................... Ajuste le pasdu Settings
          Serial.println(Msg_Setting);
          settingStep=(inputString.substring(1).toInt());
          Serial.println(settingStep);
        } else if (c==Cmd_Next) {    // ...................... Next Step
          MoveMotor(-settingStep);          
        } else if (c==Cmd_Prev) {    // ...................... Previous Step
          MoveMotor(settingStep);     
        } else if (c==Cmd_Zero) {    // ...................... Reset voie 
          Serial.print(Msg_Zero);  
          voie=0;             
        } else if (c==Cmd_Adr) {     // ...................... Adresse DCC 
          Serial.print(Msg_Adr);  
          SetCV(inputString.substring(1),Eeprom);
          Init();
          mode=1;
        } else if (c==Cmd_Track) {   // ...................... Nombre de voies 
          Serial.print(Msg_Track);  
          SetCV(inputString.substring(1),Eeprom+2);
          Init();
          mode=1;
        } else if (c==Cmd_Vitesse) { // ...................... Vitesse Maximum 
          Serial.print(Msg_Vitesse);  
          SetCV(inputString.substring(1),Eeprom+4);
          Init();
          mode=1;
        } else if (c==Cmd_Pas) {     // ...................... Pas entre voies 
          Serial.print(Msg_Pas);  
          SetCV(inputString.substring(1),Eeprom+6);
          Init();
          mode=1;
        } else if (c==Cmd_Kind) {    // ...................... Type de pont 
          Serial.print(Msg_Kind);  
          SetCV(inputString.substring(1),Eeprom+8);
          Init();
          mode=1;
        } else if (c==Cmd_Id) {         // ...................... On
          Serial.println(Msg_TT);
        } else if (c==Cmd_List) {    // ...................... Liste des parametres
          List();
        } 
        break;
    }
    
    // re-initialize la chaine recue:
    inputString = "";
    stringComplete = false;
  }

// ......................
// Fading led
  unsigned long curMillisFade = millis();
  if(curMillisFade - LastFade > FadeInterval) {
    LastFade = curMillisFade;
    if (Feu_state==true) {         // Switch on
      if (Feu_Value <(255-FadeStep)){
        Feu_Value +=FadeStep;
        analogWrite(Feu, Feu_Value); 
      }
    } else {                       // Switch off
      if (Feu_Value >0){
        Feu_Value -=FadeStep;
        analogWrite(Feu, Feu_Value); 
      }
    }
  }

// ......................
// Mouvement du moteur
  if (tmp!=NoMvt) {         // Send command to motor
    Serial.print(tmp);
    Serial.print(Msg_Motor);
    pos=pos*runingStep;
    Serial.println(pos);  
    MoveMotor(pos);
    voie=tmp;
  }

// ......................
// check goal is reached
  if (stepper.distanceToGo() == 0)  {
    if (Done) {
      Serial.print(" ... ");
      Done=false;
      if (mode==0) {
        Serial.print(Msg_Done);
        Serial.println(voie);
        digitalWrite(led1, LOW);
        digitalWrite(led2, LOW);
        Feu_state=false;
        // Enregistre la position 
        EEPROM.write(Eeprom+10, voie);
      }
      tmp=NoMvt;
    }
  } else { // detection de l'absence de moteur sur timeout
// ......................
// Clignottement leds lors du mouvement
    unsigned long curMillis = millis();
    if(curMillis - LastBlk > BlkInterval) {
      Feu_state=!Feu_state;
      LastBlk = curMillis;
      if (ledState == LOW)
        ledState = HIGH;
      else
        ledState = LOW;
      digitalWrite(led1, ledState);
      if (Alt) {
        digitalWrite(led2, ledState);
      } else {
        digitalWrite(led2, !ledState);
      }
    } 
    if(currentMillis - previousMillis > interval) {
      Connected=false;
    }
  }
  stepper.run();
 
}
